!!!!!!!!!!!!
! ft_main.f - main program for MD simulation
! Last modified: 11 August 2016
      PROGRAM argonMD
      USE mdroutines
      USE parameters
      IMPLICIT NONE
      INTEGER I, J, K
      REAL*8 pos(natoms, 3), vel(natoms, 3), force(natoms, 3)
      REAL*8 potE, kinE, totE, Temp
      
      CALL readin(pos, vel)

! Main MD loop
      DO I = 1, nsteps
         CALL calc_forces(pos, force, potE)
         CALL move_particles(pos, vel, force, kinE, Temp)
         CALL output_files(I, pos, vel,  potE, kinE, Temp)
      END DO
      END

      SUBROUTINE output_files(frame, pos, vel, potE, kinE, Temp)
      USE parameters
      IMPLICIT NONE
      INTEGER I, J, K
      INTEGER frame
      REAL*8 uc
      REAL*8 pos(natoms, 3), vel(natoms, 3)
      REAL*8 potE, kinE, Temp
      REAL*8 time, totE
      PARAMETER( uc = 10.0 )

      IF (frame .EQ. 1) THEN
         OPEN(UNIT=91, FILE='trajectory.xyz')
         WRITE(91,*) natoms
         WRITE(91,*) box*uc, box*uc, box*uc
         OPEN(UNIT=92, FILE='energy.log')
         OPEN(UNIT=93, FILE='temperature.log')
      END IF

      IF (mod((frame-1),stride) .EQ. 0) THEN
         DO I = 1, natoms
            ! convert positions to Angstroms for xyz file
            WRITE(91,*) aname, (pos(I,K)*uc, k = 1, 3)
         END DO
         time = dt * (frame - 1)
         totE = potE + kinE
         WRITE(92,*) time, potE, kinE, totE
         WRITE(93,*) time, Temp
         ! print occasional updates on simulation status
         IF (mod(frame-1,stride*10) .EQ. 0) THEN
            PRINT*, "Current time step: ", frame
         END IF
      END IF

      IF (frame .eq. nsteps) THEN
         OPEN(UNIT=94, FILE='end_structure.gro')
         WRITE(94,*) 'MD simulation of Argon'
         WRITE(94,*) nAtoms
         DO I = 1, nAtoms
         WRITE(94, 10) I, 'ARGON', 'Ar', I, (pos(I,k),k=1,3),
     :   (vel(I,k), k=1,3)
         END DO
         WRITE(94,*) box, box, box


         CLOSE(91)
         CLOSE(92)
         CLOSE(93)
         CLOSE(94)
      END IF
  10  FORMAT(i5,2a5,i5,3f8.3,3f8.4)
      END 
