!!!!!!
! ft_mdroutines.f -- contains subroutines for MD simulation
! Last modified: 11 August 2016
      MODULE mdroutines
      CONTAINS
!     Read into position arrays
      SUBROUTINE readin(pos, v)
      USE PARAMETERS
      IMPLICIT NONE
      INTEGER I, J, K
      CHARACTER(LEN=5) atomname
      INTEGER atomnum
      CHARACTER(LEN=3) resID
      REAL*8 pos
      DIMENSION pos(natoms, 3)
      REAL*8 v
      DIMENSION v(natoms, 3)
      OPEN(UNIT = 11, FILE = inputfile)
      READ(11,*)
      READ(11,*)

      DO J = 1, nAtoms
        READ(11, '(i5,2a5,i5,3f8.3,3f8.4)')atomnum,atomname,resID,
     :atomnum, (pos(J, k), k= 1, 3), (v(J, k), k = 1, 3) 
      END DO
      CLOSE(UNIT = 11)
      RETURN
      END

      SUBROUTINE calc_forces(pos, forces, PE)
      USE PARAMETERS
      IMPLICIT NONE
      INTEGER I, J, K
      INTEGER Ja, Jb
      REAL*8 pos
      DIMENSION pos(natoms, 3)
      REAL*8 forces
      DIMENSION forces(natoms, 3)
      REAL*8 dist
      DIMENSION dist(3)
      REAL*8 r_mag 
      REAL*8 PE 
      REAL*8 force_mag
      REAL*8 r6, r12 
      REAL*8 force_component
      ! Initialize force array to 0.0
         DO J = 1, nAtoms
            DO K=1, 3  
               forces(J, k) = 0.0
            END DO
         END DO
!     Calculate forces: pairwise distances between all particles
         DO Ja = 1, nAtoms - 1
            DO Jb = Ja+1, nAtoms
               DO k = 1, 3
                   dist(k) = pos(Ja, k) - pos(Jb, k)
                   dist(k) = dist(k) - ANINT(dist(k)/box)*box
               END DO          
               r_mag = 0.0
               DO k = 1,3
                  r_mag = r_mag + dist(k)**2
               END DO
               ! only compute forces if closer than cutoff
               IF (r_mag .LT. Rc2) THEN 
                  r_mag = SQRT(r_mag)
                  r6 = (sigma / r_mag)**6 
                  r12 = r6**2 
                  PE = PE + (r12 - r6) 
                  force_mag = (24.0*eps*1E9/r_mag*(2.0*r12-r6)) 
                  DO k = 1, 3
                     force_component = dist(k) * force_mag / r_mag 
                     forces(Ja,k) = forces(Ja,k) + force_component 
                     forces(Jb,k) = forces(Jb,k) - force_component 
                  END DO
               END IF
            END DO
         END DO
         PE = 4.0 * eps * PE 
      RETURN
      END
!     Leapfrog for moving particles
      SUBROUTINE move_particles(pos, v, forces, KE, sys_temp)
      USE PARAMETERS
      IMPLICIT NONE
      INTEGER I, J, K
      REAL*8 sys_temp
      REAL*8 pos
      DIMENSION pos(natoms, 3)
      REAL*8 v
      DIMENSION v(natoms, 3)
      REAL*8 forces
      DIMENSION forces(natoms, 3)
      REAL*8 KE
      REAL*8 acc
      REAL*8 dist(3) 
      REAL*8 old_v
      REAL*8 tscale
      KE = 0.0
      DO J = 1, nAtoms
         DO k = 1, 3
            old_v = v(J, k)
            acc = forces(J,k) * ( 1E-15 / mass)
            v(J, k) = old_v +  dt * acc 
            KE = KE + ( 0.5*( v(J,k) + old_v ) )**2 
            pos(J, k) = pos(J, k) + dt*v(J, k) 
         END DO
      END DO
      KE = 0.5*mass*1E6*KE 
      sys_temp = KE/(1.5*nAtoms*boltz_k)
      ! Thermostat, if temperature coupling is used.
      IF (Tcoupl .EQV. .TRUE.) THEN
         tscale = SQRT(Ref_T/sys_temp)
         DO J = 1, natoms
            DO K = 1, 3
               v(J, k) = v(J, k) * tscale
            END DO
         END DO
      END IF
      RETURN
      END
      END MODULE mdroutines
